import { Config, ModelValue, Effect, Model, QueryParams } from "./types.js";
import { getImmutableRecordKey } from "./utils.js";
export declare abstract class AbstractModelDB {
    readonly config: Config;
    static getImmutableRecordKey: typeof getImmutableRecordKey;
    readonly models: Record<string, Model>;
    constructor(config: Config);
    abstract close(): void;
    abstract get(modelName: string, key: string): Promise<ModelValue | null>;
    abstract selectAll(modelName: string): Promise<ModelValue[]>;
    abstract iterate(modelName: string): AsyncIterable<ModelValue>;
    abstract query(modelName: string, query: QueryParams): Promise<ModelValue[]>;
    abstract count(modelName: string): Promise<number>;
    set(modelName: string, key: string, value: ModelValue, options?: {
        version?: string;
    }): Promise<void>;
    delete(modelName: string, key: string, options?: {
        version?: string;
    }): Promise<void>;
    add(modelName: string, value: ModelValue, options?: {
        namespace?: string;
    }): Promise<string>;
    remove(modelName: string, key: string): Promise<void>;
    abstract apply(effects: Effect[], options: {
        namespace?: string;
        version?: string;
    }): Promise<void>;
}
