import { getImmutableRecordKey } from "./utils.js";
export class AbstractModelDB {
    constructor(config) {
        this.config = config;
        this.models = {};
        for (const model of config.models) {
            this.models[model.name] = model;
        }
    }
    async set(modelName, key, value, options = {}) {
        await this.apply([{ operation: "set", model: modelName, key, value }], { version: options.version });
    }
    async delete(modelName, key, options = {}) {
        await this.apply([{ operation: "delete", model: modelName, key }], { version: options.version });
    }
    async add(modelName, value, options = {}) {
        await this.apply([{ operation: "add", model: modelName, value }], { namespace: options.namespace });
        return getImmutableRecordKey(value, { namespace: options.namespace });
    }
    async remove(modelName, key) {
        await this.apply([{ operation: "remove", model: modelName, key }], {});
    }
}
AbstractModelDB.getImmutableRecordKey = getImmutableRecordKey;
