import { ImmutableRecordAPI, Model, ModelValue, MutableRecordAPI, RelationAPI, Resolve, TombstoneAPI } from "./types.js";
export type MutableModelDBContext = {
    tombstones: TombstoneAPI;
    relations: Record<string, RelationAPI>;
    records: MutableRecordAPI;
    model: Model;
    resolve?: Resolve;
};
export type ImmutableModelDBContext = {
    relations: Record<string, RelationAPI>;
    records: ImmutableRecordAPI;
    model: Model;
};
export declare class MutableModelAPI {
    static get(key: string, dbContext: MutableModelDBContext): Promise<ModelValue | null>;
    static set(key: string, value: ModelValue, options: {
        version?: string | null | undefined;
    } | undefined, dbContext: MutableModelDBContext): Promise<void>;
    static delete(key: string, options: {
        version?: string | null | undefined;
    } | undefined, dbContext: MutableModelDBContext): Promise<void>;
    static iterate(dbContext: MutableModelDBContext): AsyncIterable<ModelValue>;
    static selectAll(dbContext: MutableModelDBContext): Promise<ModelValue[]>;
    static query(query: {}, dbContext: MutableModelDBContext): Promise<ModelValue[]>;
    static count(dbContext: MutableModelDBContext): Promise<number>;
}
export declare class ImmutableModelAPI {
    static add(value: ModelValue, { namespace }: {
        namespace?: string | undefined;
    } | undefined, dbContext: ImmutableModelDBContext): Promise<string>;
    static remove(key: string, dbContext: ImmutableModelDBContext): Promise<void>;
    static get(key: string, dbContext: ImmutableModelDBContext): Promise<ModelValue | null>;
    static iterate(dbContext: ImmutableModelDBContext): AsyncIterable<ModelValue>;
    static selectAll(dbContext: ImmutableModelDBContext): Promise<ModelValue[]>;
    static query(query: {}, dbContext: ImmutableModelDBContext): Promise<ModelValue[]>;
    static count(dbContext: ImmutableModelDBContext): Promise<number>;
}
